export interface Gettable<V> {
    get(): V;
}
export interface Settable<V> extends Gettable<V> {
    set(v: V): void;
}
export declare class Dependency {
    listeners: Set<() => void>;
    add(listener: () => void): void;
    delete(listener: () => void): void;
    emit(): void;
}
export declare class Value<V> implements Settable<V> {
    value: V;
    dependency: Dependency;
    constructor(value: V);
    get(): V;
    set(value: V): void;
    update(fn: (v: V) => V): void;
}
export declare class DerivedValue<V> implements Gettable<V> {
    value: V;
    dependency: Dependency;
    computation: Set<Dependency>;
    fn: () => V;
    stale: boolean;
    constructor(fn: () => V);
    run(): void;
    onUpdate: () => void;
    flush(): void;
    get(): V;
    stop(): void;
}
export declare class Constraint<V> implements Settable<V> {
    value: DerivedValue<V>;
    setter: (v: V) => void;
    constructor({get, set}: {
        get: () => V;
        set: (v: V) => void;
    });
    get(): V;
    set(value: V): void;
    update(fn: (v: V) => V): void;
}
