"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Dependency = (function () {
    function Dependency() {
        this.listeners = new Set();
    }
    Dependency.prototype.add = function (listener) {
        this.listeners.add(listener);
    };
    Dependency.prototype.delete = function (listener) {
        this.listeners.delete(listener);
    };
    Dependency.prototype.emit = function () {
        this.listeners.forEach(function (listener) { return listener(); });
    };
    return Dependency;
}());
exports.Dependency = Dependency;
var computations = [];
var Value = (function () {
    function Value(value) {
        this.dependency = new Dependency();
        this.value = value;
    }
    Value.prototype.get = function () {
        var computation = computations[0];
        computation && computation.add(this.dependency);
        return this.value;
    };
    Value.prototype.set = function (value) {
        this.value = value;
        this.dependency.emit();
    };
    Value.prototype.update = function (fn) {
        this.set(fn(this.get()));
    };
    return Value;
}());
exports.Value = Value;
var DerivedValue = (function () {
    function DerivedValue(fn) {
        var _this = this;
        this.dependency = new Dependency();
        this.computation = new Set();
        this.stale = true;
        this.onUpdate = function () {
            _this.stale = true;
            _this.dependency.emit();
        };
        this.fn = fn;
    }
    DerivedValue.prototype.run = function () {
        var _this = this;
        computations.push(new Set());
        this.value = this.fn();
        var computation = computations.shift();
        this.stop();
        computation.forEach(function (dep) { return dep.add(_this.onUpdate); });
        this.computation = computation;
    };
    DerivedValue.prototype.flush = function () {
        if (this.stale) {
            this.stale = false;
            this.run();
        }
    };
    DerivedValue.prototype.get = function () {
        this.flush();
        var deps = computations[0];
        deps && deps.add(this.dependency);
        return this.value;
    };
    DerivedValue.prototype.stop = function () {
        var _this = this;
        this.computation.forEach(function (dep) { return dep.delete(_this.onUpdate); });
    };
    return DerivedValue;
}());
exports.DerivedValue = DerivedValue;
var Constraint = (function () {
    function Constraint(_a) {
        var get = _a.get, set = _a.set;
        this.value = new DerivedValue(get);
        this.setter = set;
    }
    Constraint.prototype.get = function () {
        return this.value.get();
    };
    Constraint.prototype.set = function (value) {
        this.setter(value);
    };
    Constraint.prototype.update = function (fn) {
        this.set(fn(this.get()));
    };
    return Constraint;
}());
exports.Constraint = Constraint;
function equal(x, y) {
    if (x === y) {
        return true;
    }
    if (x.size !== y.size) {
        return false;
    }
    for (var _i = 0, _a = Array.from(x); _i < _a.length; _i++) {
        var value = _a[_i];
        if (!y.has(value)) {
            return false;
        }
    }
    return true;
}
//# sourceMappingURL=reactive.js.map